/* global require */
const {
  src,
  dest,
  series,
  parallel,
  watch
} = require('gulp')
const twig = require('gulp-twig') // Decided to use twig instead of pug, because already familiar with it
const prefix = require('gulp-autoprefixer')
const concat = require('gulp-concat')
const sass = require('gulp-sass')(require('sass'))
const sourcemaps = require('gulp-sourcemaps')
const browserSync = require('browser-sync').create()
const path = require('path')
/*
 * Directories here
 */
const paths = {
  build: './build/',
  sass: './src/scss/',
  css: './build/assets/css/'
}

/**
 * Recompile files in callback and live reload the browser
 */
function browserSyncReload (cb) {
  browserSync.reload()
  cb()
}

/**
 * Wait for twig, js and sass tasks, then launch the browser-sync Server
 */
function browserSyncServe (cb) {
  browserSync.init(null, {
    notify: false,
    browser: ['chrome'],
    server: {
      baseDir: paths.build
    },
    ui: {
      port: 8080
    }
  })
  cb()
}
/**
* Compile .twig files and pass data from json file
* matching file name. index.twig - index.twig.json into HTMLs
*/
function twigTask () {
  return src(['./src/templates/*.html'])
    .pipe(twig())
    .on('error', function (err) {
      process.stderr.write(err.message + '\n')
      this.emit('end')
    }).pipe(dest(paths.build))
}

/**
 * Compile .scss files into build css directory With autoprefixer no
 * need for vendor prefixes then live reload the browser.
 */
function sassTask () {
  return src(paths.sass + 'main.scss')
    .pipe(sourcemaps.init())
    // Stay live and reload on error
    .pipe(
      sass({
        includePaths: [paths.sass + 'vendors/'],
        outputStyle: 'expanded'
      }).on('error', function (err) {
        console.log(err.message)
        this.emit('end')
      })
    )
    .pipe(prefix(['last 15 versions', '> 1%', 'ie 8', 'ie 7'], {
      cascade: true
    }))
    .pipe(sourcemaps.write('.'))
    .pipe(dest(paths.css))
}

/**
 * Compile .js files into build js directory With app.min.js
 */
function jsTask () {
  return src('build/assets/js/scripts.js')
    .pipe(sourcemaps.init())
    .pipe(concat('script.min.js'))
    .on('error', function (err) {
      console.log(err.toString())
      this.emit('end')
    })
    .pipe(sourcemaps.write('.'))
    .pipe(dest('build/assets/js'))
}

/**
 * Watch scss files for changes & recompile
 * Watch .twig files run twig-rebuild then reload BrowserSync
 */
function watchTask () {
  watch('build/assets/js/script.js').on('all', series(jsTask, browserSyncReload))
  watch('src/scss/**/*.scss').on('all', series(sassTask, browserSyncReload))
  watch('src/templates/**/*.html').on('all', series(twigTask, browserSyncReload))
}

exports.default = series(twigTask,
  parallel(sassTask, jsTask),
  series(browserSyncServe, watchTask)
)
